// x10events.cc: adjust x10 crontab according to comments
// Copyright (C) 1999 "Christpher R. Wren" <c@drwren.com>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// by "Christpher R. Wren" <c@drwren.com>
// Sun Oct 10 1999 

// { rise offset | set offset | abs }
// rise and set take offsets in decimal hours
// abs is the absolute hour and minute in the crontab field
// For example:
// 
//   22  5 * * * /usr/local/bin/heyu turn tree on        # rise +0:00
//   10 18 * * * /usr/local/bin/heyu turn tree off       # set +0.00
//   40 17 * * * /usr/local/bin/heyu turn floorlamp on   # set -0.50
//   00 00 * * * /usr/local/bin/heyu turn floorlamp off  # abs 

#include <iostream.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include "ephem.h"

main (int argc, char *argv[])
{
  if (argc != 1 && argc != 4)
    {
      cerr << "usage: "<< argv[0] << " <lat(deg) long(deg) zone(hrs)>" << endl;
      exit(1);
    }

  float latitude;
  float longitude;
  float timezone;

  if (argc > 1) latitude = atof (argv[1]);
  else latitude = 42.3778343;

  if (argc > 2) longitude = atof (argv[2]);
  else longitude = -71.1063232;

  if (argc > 3) timezone = atof (argv[3]);
  else timezone = -5;

  time_t wallclock;  time(&wallclock);
  struct tm *tmp = localtime(&wallclock);

  int Y = tmp->tm_year+1900;
  int M = tmp->tm_mon+1;
  int D = tmp->tm_mday; 
  
  int risep;
  float rise;
  float riseaz;
  int setp;
  float set;
  float setaz;
  int allday;
  
  riseset(latitude, longitude, timezone,
	  Y, M, D,
	  risep, rise, riseaz, setp, set, setaz, allday);
  
  if (tmp->tm_isdst)		// deal with daylight savings time
    {
      rise += 1;
      set += 1;
    }
  
  if (!risep)
    {
      rise = 00.00;
    }
  
  if (!setp)
    {
      set = 24.00;
    }

  unsigned int buflen = 256;
  char *buf = new char[buflen];

  while(! cin.eof())
    {
      cin.getline(buf, buflen, '\n');
      if (buf[0] != '#')
	{
	  char *command;
	  char *token;
	  char *offset;
	  char *minute;
	  char *hour;

	  minute = strtok(buf, " "); // minute
	  hour = strtok((char *) NULL, " "); // hour

	  strtok((char *) NULL, " "); // day of month
	  strtok((char *) NULL, " "); // month
	  strtok((char *) NULL, " "); // day of week

	  command = strtok((char *) NULL, "#"); // command
	  token = strtok((char *) NULL, " "); // rise,set,abs
	  offset = strtok((char *) NULL, " "); // rise,set,abs

	  if (command != (char *) NULL)
	    {
	      if (!strcmp(token, "abs"))
		{
		  cout << minute << " " << hour << " * * * " << command;
		  cout << " # abs " << endl;
		}
	      if (!strcmp(token, "rise"))
		{
		  float time = rise + atof(offset);
		  int hour = (int) floor(time);
		  int minute = (int) ((time - hour) * 60.0);
		  
		  cout << minute << " " << hour << " * * * " << command;
		  cout << " # rise " << offset << endl;
		}
	      if (!strcmp(token, "set"))
		{
		  float time = set + atof(offset);
		  int hour = (int) floor(time);
		  int minute = (int) ((time - hour) * 60.0);

		  cout << minute << " " << hour << " * * * " << command;
		  cout << " # set " << offset << endl;
		}
	    }

	}
    }

}
